@@ -118,6 +118,33 @@ impl CustomError for DecodeBodyError {
     }
 }
 
+impl CustomError for HrpcError {
+    fn error_message(&self) -> Cow<'_, str> {
+        Cow::Borrowed(&self.human_message)
+    }
+
+    fn status(&self) -> StatusCode {
+        match self.identifier.as_str() {
+            "hrpc.resource-exhausted" => StatusCode::TOO_MANY_REQUESTS,
+            "hrpc.not-implemented" => StatusCode::NOT_IMPLEMENTED,
+            "hrpc.not-found" => StatusCode::NOT_FOUND,
+            _ => StatusCode::INTERNAL_SERVER_ERROR,
+        }
+    }
+
+    fn identifier(&self) -> Cow<'_, str> {
+        Cow::Borrowed(&self.identifier)
+    }
+
+    fn more_details(&self) -> Bytes {
+        self.more_details.clone()
+    }
+
+    fn as_hrpc_error(&self) -> HrpcError {
+        self.clone()
+    }
+}
+
 /// Shorthand type for `Result<T, ServerError>.
 pub type ServerResult<T> = Result<T, ServerError>;
 
