@@ -1,4 +1,6 @@
-use std::{borrow::Cow, convert::Infallible};
+use std::{
+    borrow::Cow, convert::Infallible, mem::ManuallyDrop, ops::DerefMut, ptr::NonNull, task::Poll,
+};
 
 use super::{
     service::{not_found, CallFuture, HrpcService},
@@ -100,13 +102,21 @@ impl Routes {
     /// - This can panic if one of the paths of the inserted routes is invalid.
     pub fn build(self) -> RoutesFinalized {
         let mut matcher = Matcher::new();
+        let mut all_routes = Vec::with_capacity(self.handlers.len());
 
         for (path, handler) in self.handlers {
-            matcher.insert(path, handler).expect("invalid route path");
+            let manual_drop = ManuallyDrop::new(handler);
+            let mut_ptr = Box::leak(Box::new(manual_drop)) as *mut ManuallyDrop<HrpcService>;
+            let handler_ptr = NonNull::new(mut_ptr).unwrap();
+            matcher
+                .insert(path, handler_ptr)
+                .expect("invalid route path");
+            all_routes.push(handler_ptr);
         }
 
         let internal = RoutesInternal {
             matcher,
+            all_routes,
             any: self.any.unwrap_or_else(not_found),
         };
 
@@ -120,10 +130,23 @@ impl Routes {
 }
 
 struct RoutesInternal {
-    matcher: Matcher<HrpcService>,
+    matcher: Matcher<NonNull<ManuallyDrop<HrpcService>>>,
+    all_routes: Vec<NonNull<ManuallyDrop<HrpcService>>>,
     any: HrpcService,
 }
 
+// SAFETY: this impl is safe since our NonNull is not aliased while `Send`ing
+unsafe impl Send for RoutesInternal {}
+
+impl Drop for RoutesInternal {
+    fn drop(&mut self) {
+        for route_ptr in &mut self.all_routes {
+            // SAFETY: this is required since we are manually dropping the items
+            unsafe { ManuallyDrop::drop(route_ptr.as_mut()) }
+        }
+    }
+}
+
 impl Service<HttpRequest> for RoutesInternal {
     type Response = HttpResponse;
 
@@ -131,18 +154,25 @@ impl Service<HttpRequest> for RoutesInternal {
 
     type Future = CallFuture<'static>;
 
-    fn poll_ready(
-        &mut self,
-        _cx: &mut std::task::Context<'_>,
-    ) -> std::task::Poll<Result<(), Self::Error>> {
-        // TODO: fix this when able to get all values inside a matcher
-        Ok(()).into()
+    fn poll_ready(&mut self, cx: &mut std::task::Context<'_>) -> Poll<Result<(), Self::Error>> {
+        for route_ptr in &mut self.all_routes {
+            // SAFETY: our ptr is always initialized
+            let route = unsafe { route_ptr.as_mut() };
+            if Service::poll_ready(route.deref_mut(), cx).is_pending() {
+                return Poll::Pending;
+            }
+        }
+        Service::poll_ready(&mut self.any, cx)
     }
 
     fn call(&mut self, req: HttpRequest) -> Self::Future {
         let path = req.uri().path();
         match self.matcher.at_mut(path) {
-            Ok(matched) => Service::call(matched.value, req),
+            Ok(matched) => Service::call(
+                // SAFETY: our ptr is always initialized
+                unsafe { matched.value.as_mut().deref_mut() },
+                req,
+            ),
             Err(err) if err.tsr() => {
                 let redirect_to = if let Some(without_tsr) = path.strip_suffix('/') {
                     Cow::Borrowed(without_tsr)
@@ -150,7 +180,13 @@ impl Service<HttpRequest> for RoutesInternal {
                     Cow::Owned(format!("{}/", path))
                 };
                 match self.matcher.at_mut(redirect_to.as_ref()) {
-                    Ok(matched) => Service::call(matched.value, req),
+                    Ok(matched) => {
+                        Service::call(
+                            // SAFETY: our ptr is always initialized
+                            unsafe { matched.value.as_mut().deref_mut() },
+                            req,
+                        )
+                    }
                     _ => Service::call(&mut self.any, req),
                 }
             }
