@@ -1,8 +1,6 @@
-use std::{net::ToSocketAddrs, time::Duration};
-
 use crate::HttpRequest;
 
-use super::{HrpcLayer, Service};
+use super::HrpcLayer;
 
 use http::{header::HeaderName, HeaderMap};
 use tower::ServiceBuilder;
@@ -32,39 +30,6 @@ impl HeaderMapExt for HeaderMap {
     }
 }
 
-/// Start serving a server on the specified address.
-///
-/// If given multiple addresses, it will try serving on each address and if
-/// all of them returns an error, it will return the last error.
-pub async fn serve<A, S>(server: S, address: A) -> Result<(), hyper::Error>
-where
-    A: ToSocketAddrs,
-    S: Service,
-{
-    let mut addrs = address
-        .to_socket_addrs()
-        .expect("could not convert to socket address");
-
-    let mut successful_addr = addrs.next().expect("no socket address provided");
-    let mut builder = hyper::Server::try_bind(&successful_addr);
-    for addr in addrs {
-        builder = if builder.is_err() {
-            successful_addr = addr;
-            hyper::Server::try_bind(&successful_addr)
-        } else {
-            break;
-        };
-    }
-
-    let server = builder?
-        .http1_keepalive(true)
-        .http2_keep_alive_interval(Some(Duration::from_secs(10)))
-        .http2_keep_alive_timeout(Duration::from_secs(20))
-        .serve(server.into_make_service());
-    tracing::info!("serving at {}", successful_addr);
-    server.await
-}
-
 /// A set of "recommended" layers, namely:
 /// - [`TraceLayer`] for tracing requests, enables adding headers as fields by default
 ///     - pass `|_| true` if you don't want to filter any headers
